# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zwave_me_ws']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0', 'websocket-client>=1.2.1,<2.0.0']

setup_kwargs = {
    'name': 'zwave-me-ws',
    'version': '0.4.1',
    'description': 'Library, implementing websocket connection to ZWave-Me',
    'long_description': 'ZWave-Me-WS is a websocket connection library for [Z-Wave.Me Z-Way](https://z-wave.me/z-way/) controllers\n\n**Installing**\n\nTo install this package use:\n\n`pip install zwave-me-ws`\n\n**Usage**\n\nTo use the connector initialize the API using:\n\n```\nfrom zwave_me_ws import ZWaveMe, ZWaveMeData\n\nzwave_api = ZWaveMe(\n  on_device_create=on_device_func\n  on_device_update=on_device_update_func,\n  on_new_device=on_device_add_func,\n  token="....", # Z-Way access token in the form .../... (remote) of /... (local)\n  url="...", # wss://find.z-wave.me or ws://IP:8083\n  platforms=[...]\n)\n\ndef on_device_add_func(self, device: ZWaveMeData)\n\ndef on_device_create_func(self, devices: list[ZWaveMeData])\n\ndef on_device_update_func(self, new_info: ZWaveMeData)\n```\n\nHere `platforms` is the list of deviceType fields to handle. Used to filter only types supported by your application. For example, \n`["sensorBinary", "lightMultilevel", "toggleButton", "thermostat", "motor", "fan", "doorlock", "switchMultilevel", "switchBinary", "sensorMultilevel", "siren", "switchRGBW", "switchRGB"]`.\n\nAvailable functions:\n```\ndevices = zwave_api.get_devices()\n\nzwave_api.send_command(device_id, command) # command can be "on", "off", "exact?level=..", "open", "close"\n\nis_connected = zwave_api.get_connection()\n\nzwave_api.close_ws()\n```\n\nDevice (ZWaveMeData) has the following fields:\n```\n  id: str\n  deviceType: str\n  title: str\n  level: Union[str, int, float]\n  deviceIdentifier: str\n  probeType: str\n  scaleTitle: str\n  min: str\n  max: str\n  color: dict\n  isFailed: bool\n  locationName: str\n  manufacturer: str\n  firmware: str\n  tags: list[str]\n  nodeId: str\n  creatorId: str\n```\n',
    'author': 'Dmitry Vlasov',
    'author_email': 'kerbalspacema@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Z-Wave-Me/zwave-me-ws',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass, field
from typing import Union

FIELDS = [
    "id",
    "deviceType",
    "probeType",
    "locationName",
    "manufacturer",
    "firmware",
    "tags",
    "creatorId",
    "nodeId",
]
METRICS_SCALE = ["title", "level", "scaleTitle", "min", "max", "color", "isFailed"]

TYPE_TAGS = {
    'type-sensor-binary': "sensorBinary",
    'type-light': "lightMultilevel",
    'type-button': "toggleButton",
    'type-thermostat': "thermostat",
    'type-motor': "motor",
    'type-fan': "fan",
    'type-doorlock': "doorlock",
    'type-number': "switchMultilevel",
    'type-switch': "switchBinary",
    'type-sensor': "sensorMultilevel",
    'type-siren': "siren",
}


@dataclass
class ZWaveMeData:
    id: str
    deviceType: str
    title: str
    level: Union[str, int, float]
    deviceIdentifier: str
    probeType: str = ""
    scaleTitle: str = ""
    min: str = ""
    max: str = ""
    color: dict = field(default_factory=dict)
    isFailed: bool = False
    locationName: str = ""
    manufacturer: str = ""
    firmware: str = ""
    tags: list[str] = field(default_factory=list)
    nodeId: str = ""
    creatorId: str = ""


def prepare_devices(devices: list[dict]) -> list[ZWaveMeData]:
    prepared_devices = []

    for device in devices:
        if device['permanently_hidden']:
            continue
        prepared_device = {
            **{key: device[key] for key in FIELDS if key in device},
            **{
                key: device["metrics"][key]
                for key in METRICS_SCALE
                if key in device["metrics"]
            },
        }
        prepared_device = set_device_type(prepared_device)
        if prepared_device["deviceType"] == "motor":
            if prepared_device["level"] == "off":
                prepared_device["level"] = 0
            if prepared_device["level"] == "on":
                prepared_device["level"] = 99.0
            prepared_device["level"] = float(prepared_device["level"])

        if "creatorId" in prepared_device and "nodeId" in prepared_device:
            prepared_device[
                "deviceIdentifier"
            ] = f"{prepared_device['creatorId']}_{prepared_device['nodeId']}"
        else:
            prepared_device["deviceIdentifier"] = prepared_device["id"]

        prepared_devices.append(prepared_device)

    return [ZWaveMeData(**d) for d in prepared_devices]


def set_device_type(prepared_device):
    if prepared_device["probeType"] == "siren":
        prepared_device["deviceType"] = "siren"
    if prepared_device["tags"]:
        for tag in prepared_device["tags"]:
            if tag in TYPE_TAGS:
                prepared_device["deviceType"] = TYPE_TAGS[tag]
                prepared_device = set_value_by_device_type(prepared_device)
                return prepared_device
    if prepared_device["probeType"] == "motor":
        prepared_device["deviceType"] = "motor"
    elif prepared_device["probeType"] == "fan":
        prepared_device["deviceType"] = "fan"
    return prepared_device


def set_value_by_device_type(prepared_device) -> dict:
    if prepared_device['deviceType'] == "sensorBinary":
        if prepared_device['level'] in ('on', 'off'):
            return prepared_device
        elif prepared_device['level'] in ('open', 'close'):
            prepared_device['level'] = {'open': 'off', 'close': 'on'}[prepared_device['level']]
        else:
            prepared_device['level'] = 'on' if bool(prepared_device['level']) else 'off'
    elif prepared_device['deviceType'] == 'lightMultilevel':
        if str(prepared_device['level']).replace('.', '', 1).isdigit():
            prepared_device["color"] = {
                "r": round(2.55 * float(prepared_device["level"])),
                "g": round(2.55 * float(prepared_device["level"])),
                "b": round(2.55 * float(prepared_device["level"])),
            }
            prepared_device["level"] = (
                "on" if float(prepared_device["level"]) > 0 else "off"
            )
        elif prepared_device['level'] == 'on':
            prepared_device["color"] = {
                "r": 255,
                "g": 255,
                "b": 255,
            }
        elif prepared_device['level'] == 'off':
            prepared_device["color"] = {
                "r": 0,
                "g": 0,
                "b": 0,
            }
        else:
            prepared_device['level'] = 'on' if bool(prepared_device['level']) else 'off'
            if prepared_device['level'] == 'on':
                prepared_device["color"] = {
                    "r": 255,
                    "g": 255,
                    "b": 255,
                }
            elif prepared_device['level'] == 'off':
                prepared_device["color"] = {
                    "r": 0,
                    "g": 0,
                    "b": 0,
                }
    elif prepared_device['deviceType'] == 'toggleButton':
        return prepared_device
    elif prepared_device['deviceType'] == 'thermostat':
        if str(prepared_device['level']).replace('.', '', 1).isdigit():
            return prepared_device
        elif prepared_device['level'] == 'on':
            prepared_device['level'] = 99
        elif prepared_device['level'] == 'off':
            prepared_device['level'] = 0
        else:
            prepared_device['level'] = 99 if bool(prepared_device['level']) else 0
    elif prepared_device['deviceType'] == 'motor':
        if str(prepared_device['level']).replace('.', '', 1).isdigit():
            return prepared_device
        elif prepared_device['level'] == 'on':
            prepared_device['level'] = 99
        elif prepared_device['level'] == 'off':
            prepared_device['level'] = 0
        else:
            prepared_device['level'] = 99 if bool(prepared_device['level']) else 0
    elif prepared_device['deviceType'] == 'fan':
        if str(prepared_device['level']).replace('.', '', 1).isdigit():
            return prepared_device
        elif prepared_device['level'] == 'on':
            prepared_device['level'] = 99
        elif prepared_device['level'] == 'off':
            prepared_device['level'] = 0
        else:
            prepared_device['level'] = 99 if bool(prepared_device['level']) else 0
    elif prepared_device['deviceType'] == 'doorlock':
        if prepared_device['level'] in ('open', 'close'):
            return prepared_device
        elif prepared_device['level'] in ('on', 'off'):
            prepared_device['level'] = {'off': 'open', 'on': 'close'}[prepared_device['level']]
        else:
            prepared_device['level'] = 'close' if bool(prepared_device['level']) else 'on'

    elif prepared_device['deviceType'] == 'switchMultilevel':
        if str(prepared_device['level']).replace('.', '', 1).isdigit():
            return prepared_device
        elif prepared_device['level'] == 'on':
            prepared_device['level'] = 99
        elif prepared_device['level'] == 'off':
            prepared_device['level'] = 0
        else:
            prepared_device['level'] = 99 if bool(prepared_device['level']) else 0
    elif prepared_device['deviceType'] == 'switchBinary':
        if prepared_device['level'] in ('on', 'off'):
            return prepared_device
        elif prepared_device['level'] in ('open', 'close'):
            prepared_device['level'] = {'open': 'off', 'close': 'on'}[prepared_device['level']]
        else:
            prepared_device['level'] = 'on' if bool(prepared_device['level']) else 'off'
    elif prepared_device['deviceType'] == 'sensorMultilevel':
        if str(prepared_device['level']).replace('.', '', 1).isdigit():
            return prepared_device
        elif prepared_device['level'] == 'on':
            prepared_device['level'] = 99
        elif prepared_device['level'] == 'off':
            prepared_device['level'] = 0
        else:
            prepared_device['level'] = 99 if bool(prepared_device['level']) else 0
    elif prepared_device['deviceType'] == 'siren':
        if prepared_device['level'] in ('on', 'off'):
            return prepared_device
        elif prepared_device['level'] in ('open', 'close'):
            prepared_device['level'] = {'open': 'off', 'close': 'on'}[prepared_device['level']]
        else:
            prepared_device['level'] = 'on' if bool(prepared_device['level']) else 'off'
    return prepared_device
